#!/usr/bin/python3
"""
Interceptor for dh-fortran targets

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 GNU Public License.
"""

import sys
from os.path import basename, islink
from functools import wraps
import dhfortran.cli as cli
import dhfortran.compilers as cmplrs
import dhfortran.targets as tgt
import click


def foreach_compiler(fail_on_error_on_default=True):
    """ Wrapper. Execute the function on all compilers """
    def outer(f):
        @wraps(f)
        def inner(*args, **kwargs):
            cmd  = basename(sys.argv[0])
            try:
                for flavor in cmplrs.get_fc_required():
                    print(f"   {cmd} ( {flavor} env)")
                    f(flavor, cmd, *args, **kwargs)
            except Exception:
                if fail_on_error_on_default:
                    raise
            for flavor in cmplrs.get_fc_optional():
                 try:
                     print(f"   {cmd} ( {flavor} env)")
                     f(flavor,cmd, *args, **kwargs)
                 except Exception as ex:
                     cli.verbose_print(f"{cmd}: ignoring error '{ex}'"
                                       "(optional compiler)")
        return inner
    return outer




@foreach_compiler()
def dh_auto_with_flavor_env(flavor: str,cmd: str, *args, **kwargs):
    tgt.set_env(flavor)
    # Not sure the following happen
    if (" ".join(sys.argv[1:])).find('-O--builddirectory') != -1:
        cmdlist = [ f"/usr/bin/{cmd}" ] + sys.argv[1:]
    else:
        builddir = f"-O--builddirectory={kwargs['builddir']}-{flavor}"
        # TODO: calling this too early breaks things; drop tmpdir ?
        # Set only in install case ?
        # tmpdir = f"--tmpdir={kwargs['tmpdir']}-{flavor}"
        cmdlist  = [ f"/usr/bin/{cmd}", builddir] + sys.argv[1:]  
    tgt.doit(cmdlist)
    

def noop(*args, **kwargs):
    """ Helper. For now, just redirect back to /usr/bin/dh_auto_* """
    cmd  = basename(sys.argv[0])
    cmdlist  = [ f"/usr/bin/{cmd}" ] + sys.argv[1:]
    tgt.doit(cmdlist)

@click.command(
    context_settings=dict(
        ignore_unknown_options=True,
    )
)
@click.argument('args', nargs=-1, type=click.UNPROCESSED)
@cli.debhelper_common_args
def dh_fortran_clean(*args,**kwargs):
    """ Hijack and monkey-patch dh_auto_clean calls so we can set environment """
    if kwargs['buildsystem'] == 'fortran':
        tgt.doit(["fpm", "clean", "--all"],verbose=True)
    if kwargs['builddir'] is not None and islink(kwargs['builddir']):
        tgt.doit(['rm','-f',kwargs['builddir']])
    tgt.dh_fortran_clean(**kwargs)


@click.command(
    context_settings=dict( ignore_unknown_options=True,)
    )
@click.argument('args', nargs=-1, type=click.UNPROCESSED)
@cli.debhelper_common_args
def dh_auto_configure(*args, **kwargs):
    """ Hijack and monkey-patch dh_auto_configure calls so we can set environment """
    if kwargs['buildsystem'] == 'fortran':
        tgt.dh_fortran_configure(**kwargs)
    else:
        if cmplrs.is_multifortran_build():
            d = cmplrs.get_fc_default()
            if kwargs['builddir'] == 'debian/build':
                # Avoid incorrect symlink
                tgt.doit(['ln','-s',f"build-{d}","debian/build"])
            else:
                tgt.doit(['ln','-s',
                          f"{kwargs['builddir']}-{d}",
                          f"{kwargs['builddir']}"])
            dh_auto_with_flavor_env(*args, **kwargs)
        else:
            noop(*args, **kwargs)
            

@cli.debhelper_common_args
@click.argument('args', nargs=-1, type=click.UNPROCESSED)
def dh_auto_build(*args,**kwargs):
    """ Hijack and monkey-patch dh_auto_build calls so we can set environment """
    if kwargs['buildsystem'] == 'fortran':
        tgt.dh_fortran_build(**kwargs)
    else:
        if cmplrs.is_multifortran_build():
            dh_auto_with_flavor_env(*args, **kwargs)
        else:
            noop(*args, **kwargs)

@click.command(
    context_settings=dict(
        ignore_unknown_options=True,
    )
)
@cli.debhelper_common_args
@click.argument('args', nargs=-1, type=click.UNPROCESSED)
def dh_auto_install(*args,**kwargs):
    """ Hijack and monkey-patch dh_auto_install calls so we can set environment """ 
    match kwargs['buildsystem']:
        case 'fortran':
            tgt.dh_fortran_install(**kwargs)
        case 'cmake+makefile' | 'ecbuild+makefile':
            if cmplrs.is_multifortran_build():
                tgt.dh_install_on_cmake_makefile(**kwargs)
            noop(*args, **kwargs)
        case _:
            if cmplrs.is_multifortran_build():
                cli.verbose_print(f"Can't handle buildsystem {kwargs['buildsystem']};"
                                  "reverting to default behaviour")
            noop(*args, **kwargs)


@click.command(
    context_settings=dict(
        ignore_unknown_options=True,
    )
)
@click.argument('args', nargs=-1, type=click.UNPROCESSED)
@cli.debhelper_common_args
def dh_auto_test(*args,**kwargs):
    """ Hijack and monkey-patch dh_auto_test calls so we can set environment """
    if kwargs['buildsystem'] == 'fortran':
        tgt.dh_fortran_test(**kwargs)
    else:
        if cmplrs.is_multifortran_build():
            dh_auto_with_flavor_env(*args, **kwargs)
        noop(*args, **kwargs)

    
    



#!/usr/bin/env python3
"""
CMake fragment handling for dh_fortran

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 GNU Public License.
"""

import click
import dhfortran.debhelper as dh
import dhfortran.compilers as cmplrs
import dhfortran.cli as cli


# TODO:
# Look for INTERFACE_INCLUDE_DIRECTORIES
# Edit file (typically XXX-targets.cmake to XXX-$vendor-targets.cmake
#


# Actions we can do:
# rename references to Fortran libs ( libfckit.so -> libfckit-gfortran.so)
# rename FMODDIR


##
## CMake-file -specific interface for debhelper
##
class CMakeFileHelper(dh.DhFortranHelper):

    def __init__(self, options):
        super().__init__(options, "fortran-cmake", "dh_fortran_cmake")

    def can_work_automatically(*cmdline_files: list[str]) -> bool:
        """Called by self.process_and_move_files()
        Can we run automatically, without user configuration?

        If this is true, d.autogenerate_pkg_filelist() needs to be defined
        """
        return False

    def compute_dest(self, cmake_file, target_pkg,  target_dest=None):
        """Where does cmake file go ? Should be called by base
        DebHelper() to move files"""
        cli.debug_print("compute_dest [cmake] {cmake_file=} {target_pkg=} {target_dest=}")

        # TODO : unsafe for cross-compilation. Build our own flibdir
        flibdir = f"/usr/lib/{cmplrs.multiarch}/fortran/cmake"

        if cmake_file is None:
            return flibdir
        if cmake_file.startswith("/"):
            return cmake_file
        else:
            return f"{flibdir}/{cmake_file}"

    def process_file(self, pkg, cmake_file, target_pkg, target_dest=None):
        cli.debug_print(
            f"process_file [lib]  name {cmake_file} {target_pkg} {target_dest}"
        )
        ...


def rewrite_cmake(oldname, newname, cb_dict):
    """Rewrite a cmake file:

    Called by clone_and_rewrite():

    """
    # c
    # INTERFACE_INCLUDE_DIRECTORIES
    #   sed -i -e 's%$${_IMPORT_PREFIX}/lib/libfckit%${LIBDIR}/libfckit%g'
    #            $(TMPDIR)/$(LIBDIR)/cmake/fckit/fckit-targets.cmake
    #   sed -i -e 's%$${_IMPORT_PREFIX}/include%/usr/include/${DEB_HOST_MULTIARCH}%g' \
    # $(TMPDIR)/$(LIBDIR)/cmake/fckit/fckit-targets.cmake
    #   sed -i -e 's%$${_IMPORT_PREFIX}/module%${FMODDIR}%g'         \
    # $(TMPDIR)/$(LIBDIR)/cmake/fckit/fckit-targets.cmake
    cli.verbose_print(f"rewrite_cmake({oldname},{newname} --simplistic form")
    with open(oldname, "r") as f:
        with open(newname, "w") as n:
            n.write(f.read)


@click.command(
    context_settings=dict(
        ignore_unknown_options=False,
    )
)
@click.option("--fc", help="Fortran compiler flavor, eg. gfortran-15", default=None)
@click.argument("files", nargs=-1, type=click.UNPROCESSED)
@cli.debhelper_common_args
def dh_fortran_cmake(files, *args, **kwargs):
    """CMake file handler for dh-fortran

    *dh_fortran_cmake* is a Debhelper plugin script that may be called in the
    installation sequence to update CMake scripts.
    """
    cli.debug_print(f"dh_fortran_cmake called with files {files} kwargs {kwargs}")

    # Get defaults if not defined
    flavor, host_arch = cmplrs.get_fc_flavor_arch(kwargs["fc"])
    kwargs.update(
        {
            "flavor": flavor,
            "vendor": cmplrs.get_abi_vendor(flavor),
        }
    )
    d = CMakeFileHelper(dh.build_options(**kwargs))
    searchdirs = [d.options.tmpdir, "."]
    d.process_and_move_files(searchdirs, *files)


if __name__ == "__main__":
    import pytest

    pytest.main(["tests/cmake.py"])



#!/usr/bin/python3
"""
Driver for dh-fortran d/rules targets

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 GNU Public License.
"""

import os
import subprocess
import sys
from functools import wraps
import dhfortran.cli as cli
import dhfortran.compilers as cmplrs


def set_env(flavor):
    """Set Fortran env"""
    env = (
        subprocess.check_output(["dh_fortran", "get_env", "--fc", flavor])
        .strip()
        .decode("utf-8")
    )
    for line in env.split('\n'):
        key, val = line[: line.find("=")], line[line.find("=") + 1 :]
        os.environ[key] = val



def doit(args, verbose=False):
    if verbose and "DH_VERBOSE" in os.environ:
        args += ["--verbose"]
    cli.verbose_print(args)
    subprocess.run(args, stdout=sys.stdout, stderr=sys.stderr)


def foreach_compiler(fail_on_error_on_default=True):

    def outer(f):
        @wraps(f)
        def inner(*args, **kwargs):
            try:
                for flavor in cmplrs.get_fc_required():
                    cli.verbose_print(f"   {f.__name__} {flavor} ")
                    f(flavor=flavor, **kwargs)
            except Exception as ex:
                cli.verbose_print(f"FAIL: {f.__name__} : error {ex}")
                if fail_on_error_on_default:
                    raise
            for flavor in cmplrs.get_fc_optional():
                 try:
                     cli.verbose_print(f"   {f.__name__} {flavor} ")
                     f(flavor=flavor,  **kwargs)
                 except Exception as ex:
                     cli.verbose_print(f"{f.__name__}: ignoring error {ex}"
                                       "(testing optional compiler")
        return inner

    return outer


@foreach_compiler(fail_on_error_on_default=False)
def dh_fortran_clean(flavor,  **kwargs):
    if kwargs['builddir'] not in {None, '.'}:
            doit(["rm", "-rf", f"{kwargs['builddir']}-{flavor}"])
    if 'tmpdir' in kwargs and kwargs['tmpdir'] is not None:
        doit(["rm", "-rf", f"{kwargs['tmpdir']}-{flavor}"])


@foreach_compiler(fail_on_error_on_default=False)
def dh_fortran_install(flavor: str, **kwargs):
    """Do Fortran-specific operations on d/rules install target"""
    set_env(flavor)
    if kwargs['destdir'] is not None:
        destdir = kwargs["destdir"]
    else:
        destdir = f"{kwargs['tmpdir']}-{flavor}"
        
    # TODO If flavor = FC_DEFAULT, set a symlink for tmpdir
    if kwargs['buildsystem'] == 'fortran':
        doit(["fpm", "install", f"--prefix={destdir}/usr"],verbose=True)
    # Execute these with tmpdir set; paths in config files may be
    # relative to tmpdir
    doit(["dh_fortran_mod", f"--tmpdir={destdir}"])
    doit(["dh_fortran_lib", f"--tmpdir={destdir}",f"--fc={flavor}"])
    doit(["dh_fortran_pkgconf", f"--tmpdir={destdir}",f"--fc={flavor}"])
    doit(["dh_fortran_cmake", f"--tmpdir={destdir}",f"--fc={flavor}"])        

@foreach_compiler()
def dh_install_on_cmake_makefile(flavor: str, **kwargs):
    """ On cmake+makefile, for each Fortran-build,
    jump in and do make install with overridden destdir """
    builddir= f"{kwargs['builddir']}-{flavor}"
    if kwargs['destdir'] is not None:
        destdir = kwargs["destdir"]
    else:
        destdir = f"{kwargs['tmpdir']}-{flavor}"
    dest = f"{os.getcwd()}/{destdir}"
    doit(["make","-C", builddir,"install",f"DESTDIR={dest}"])
    doit(["dh_fortran_mod", f"--tmpdir={dest}"])
    doit(["dh_fortran_lib", f"--tmpdir={dest}",f"--fc={flavor}"])
    doit(["dh_fortran_pkgconf", f"--tmpdir={dest}"])
    doit(["dh_fortran_cmake", f"--tmpdir={dest}"])        
    

@foreach_compiler(fail_on_error_on_default=False)
def dh_fortran_configure(flavor: str, **kwargs):
    """Do Fortran-specific operations on d/rules configure target"""
    # FPM does not have a configure target
    pass


@foreach_compiler(fail_on_error_on_default=False)
def dh_fortran_build(flavor: str, **kwargs):
    """Do Fortran-specific operations on d/rules build target"""
    if kwargs['buildsystem'] == 'fortran':
        set_env(flavor)
        doit(["fpm", "build"],verbose=True)


@foreach_compiler()
def dh_fortran_test(flavor: str, **kwargs):
    """Do Fortran-specific operations on d/rules test target"""
    if kwargs['buildsystem'] == 'fortran':
        set_env(flavor)
        doit(["fpm", "test"])

if __name__ == "__main__":
    import pytest

    pytest.main(["tests/targets.py"])



#!/usr/bin/python3
"""
Module file handling for Fortran: tests

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

import dhfortran.module as mod
from click.testing import CliRunner
from pathlib import Path
import pytest

# TODO test smod files handled

# TODO test sourcedir handled /ignored

# TO test tmpdir honoured

def test_dh_fortran_mod_cmdline1():
    runner = CliRunner()
    result = runner.invoke(mod.dh_fortran_mod, ['tests/test01/debian/tmp/lib/points.mod'])
    assert result.exit_code == 0

def test_dh_fortran_mod():
    """ Install mod files. Normal case"""
    runner = CliRunner()
    result = runner.invoke(mod.dh_fortran_mod, [])
    assert result.exit_code == 0

    
def test_detection_flang_21():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/flang-21/points.mod")
    assert (cmplr,version) == ("flang","1")

def test_which_compiler_gf14():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/gfortran-14/points.mod")
    assert (cmplr, version) == ("gfortran", "15")


def test_which_compiler_gf15():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/gfortran-15/points.mod")
    assert (cmplr, version) == ("gfortran", "16")


def test_which_compiler_gf15():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/gfortran-16/points.mod")
    assert (cmplr, version) == ("gfortran", "16")

    
def test_which_compiler_fn18():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/flang-new-18/points.mod")
    assert (cmplr, version) == ("flang","1")


def test_which_compiler_fn19():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/flang-new-19/points.mod")
    assert (cmplr, version) == ("flang","1")


def test_which_compiler_fn20():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/flang-new-20/points.mod")
    assert (cmplr, version) == ("flang","1")


def test_which_compiler_fn21():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/flang-21/points.mod")
    assert (cmplr, version) == ("flang","1")

def test_which_compiler_fn22():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/flang-22/points.mod")
    assert (cmplr, version) == ("flang","1")

def test_which_compiler_lfortran():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/lfortran/points.mod")
    assert cmplr == "lfortran"
    assert version.startswith("0.")

def test_which_compiler_intel():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/intel/points.mod")
    assert (cmplr, version) == ("intel","0")

def test_which_compiler_nvhpc():
    (cmplr, version) = mod.which_compiler_flavor_and_mod_version("modfiles/nvhpc/points.mod")
    assert (cmplr,version) == ("nvhpc","35")


if __name__ == "__main__":
    import pytest

    pytest.main()



#!/usr/bin/env python3
"""

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 GNU Public License.
"""


import click
import dhfortran.debhelper as dh
import dhfortran.cli as cli
import dhfortran.compilers as cmplrs
from os.path import basename,exists


##
## Pkgconfig-file -specific interface for debhelper
##
class PkgconfFileHelper(dh.DhFortranHelper):

    def __init__(self, options):
        super().__init__(options, "fortran-pkgconf", "dh_fortran_pkgconf")

    def can_work_automatically(self, *cmdline_files: list[str]) -> bool:
        """Called by Debhelper.process_and_move_files()
        Can we run automatically, without user configuration?

        If this is true, d.autogenerate_pkg_filelist() needs to be defined
        """
        return False

    def compute_dest(self, src_file, target_pkg, target_dest=None) -> str:
        """Where does pkgconfig file go ?
        Should be called by base DebHelper() to move files"""
        cli.debug_print(f"compute_dest [pkgconf]  {src_file=} {target_dest=}")
        pkgdir = f"debian/{target_pkg}/usr/lib/{cmplrs.multiarch}/pkgconfig"

        if target_dest is None or target_dest=='':
            return pkgdir
        if target_dest.startswith("/"):
            return target_dest
        else:
            return f"{pkgdir}/{target_dest}"
        
        
        
        
    def process_file(self, pkg, filename, target_pkg, target_dest=None):
        cli.debug_print(
            f"process_file [pkgconf]  {filename=} {target_pkg=} {target_dest=}"
        )
        fmoddir = cmplrs.get_fmoddir(self.options.flavor, self.options.arch)
        flibdir = cmplrs.get_flibdir(self.options.flavor, self.options.arch)
        fc = self.options.flavor

        update = {"@FMODDIR@": fmoddir, "@FLIBDIR@": flibdir, "@FC@": fc,
                  "@DEB_HOST_MULTIARCH@" : cmplrs.multiarch}
        updated = set()
        
        destdir=f"{self.compute_dest(filename, target_pkg, target_dest)}"
        b = basename(filename)
        root = b[:b.rfind('.pc')]
        abi = cmplrs.get_abi_vendor(flavor=cmplrs.get_fc_flavor_arch(fc)[0])
        destfile = f"{destdir}/{root}-{fc}.pc"
        if exists(destfile):
            if self.options.flavor == self.options.preferred:
                cli.verbose_print(f"{destfile} exists but will be overwritten"
                "(preferred)")
            else:
                cli.verbose_print(f"{destfile} exists and will not be overwritten")
                return            
                
        with open(filename, "r") as f:
            lines = f.readlines()
        for i, line in enumerate(lines):
            for v in update:
                lines[i] = lines[i].replace(v, update[v])
                updated.add(v)
        # Updating vars is more risky. Update only if subst not already done
        for i, line in enumerate(lines):
            if line.startswith("fmoddir="):
                if "{prefix}" in line and "@FMODDIR@" in updated:
                    continue
                lines[i] = f"fmoddir={fmoddir}\n"
            if line.startswith("flibdir="):
                if "{prefix}" in line and "@FLIBDIR@" in updated:
                    continue
                lines[i] = f"flibdir={flibdir}\n"
            if line.startswith("FC="):
                if "{prefix}" in line and "@FC@" in updated:
                    continue
                lines[i] = f"FC={fc}"

        dh.install_dir(destdir)
        cli.verbose_print(f"dh_fortran_pkgconf: creating {destfile}")
        with open(destfile, "w") as n:
            for line in lines:
                print(line, file=n, end="")
        # symlinks
        dh.install_dir(f"debian/{target_pkg}{flibdir}/pkgconfig")
        dh.make_symlink(f"debian/{target_pkg}{flibdir}/pkgconfig/{root}.pc",
                        destfile,
                        target_dest)
        if self.options.flavor == self.options.preferred:            
            breakpoint()
            dh.make_symlink(f"{destdir}/{root}.pc",
                            destfile,
                            target_dest)
            

@click.command(
    context_settings=dict(
        ignore_unknown_options=False,
    )
)
@click.option("--fc", help="Fortran compiler flavor, eg. gfortran-15")
@click.option(
    "--preferred", help="Preferred compiler flavor when installing, eg gfortran-15"
)
@click.argument("files", nargs=-1, type=click.UNPROCESSED)
@cli.debhelper_common_args
def dh_fortran_pkgconf(files, *args, **kwargs):
    """
    *dh_fortran_pkgconf* is a debhelper program that rewrites pkgconfig files and
    updates paths to FMODDIR and FLIBDIR to find the correct Fortran modules files
    and libraries for a given compiler.

    *dh_fortran_pkgconf* is expected to be automatically added using the debhelper
    "addon" <fortran> ie. either automatically, by build-depending on 
    'dh-sequence-fortran', or explicitly:

        dh $@ --with fortran

    <dh_fortran_pkgconf> Searches the debian/ directory for files 
    B<debian/pkg.fortran-pkgconf> which lists pkgconfig files to process, 
    with the same syntax as debhelper install files.

    OPTIONS
        --tmpdir=dir
        --sourcedir=dir

    Look in the specified directory for files to be installed. It will look in 
    tmpdir first (typically debian/tmp) then sourceddir (.).

    USAGE

    The source pkgconfig file will be read and any placeholders @FMODDIR@, 
    @FLIBDIR@ and @FC@ will be updated to the correct values. 
    If these are not found, relevant lines in the value section will be 
    updated (eg. fmoddir= lines).

    TODO

    (1) relative entries such as prefix defines may be broken by moving pc file
    (2) Insert fmoddir, flibdir if missing


    The fortran-pkgconf file syntax follows dh_install: pairs of sources 
    and optional target directories. If the target directory is absolute 
    (starts with a  '/'), this directory is used in the target package.
    If the target  does not absolute, it will be treated as a subdirectory 
    of the pkgconfig path.

    The rewritten pkgconfig files will by default be added to 
          /usr/lib/$multiarch/fortran/$compiler-flavor/pkgconfig
    (eg usr/lib/aarch64-linux-gnu/fortran/gfortran-15/pkgconfig/mypkg.pc )
    This can then be used in the PKG_CONFIG_PATH
    """

    cli.debug_print(f"dh_fortran_pkgconf called with {kwargs}")
    fc = kwargs["fc"]
    flavor, arch = cmplrs.get_fc_flavor_arch(fc)
    kwargs.update(
        {
            "flavor": flavor,
            "arch": arch,
            "preferred": cmplrs.get_preferred(kwargs["preferred"], flavor),
        }
    )
    d = PkgconfFileHelper(dh.build_options(**kwargs))
    seachdirs = [d.options.tmpdir, "."]
    d.process_and_move_files(seachdirs, *files)
    ...


if __name__ == "__main__":
    import pytest

    pytest.main(["tests/pkgconf.py"])



#!/usr/bin/python3
"""
CMake fragment handling for Fortran : tests

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

import dhfortran.debhelper as dh
import os
import pytest
from dhfortran.cmake import CMakeFileHelper


@pytest.fixture()
def set_env():
    os.environ["NO_ACT"] = "1"


def test_debhelper_initialization():
    opts = dh.build_options()
    c = CMakeFileHelper(opts)


# @pytest.mark.xfail
# def test_clone_and_rename(set_env):
#     assert (
#         dh.clone_and_rename("hipsparse-targets.cmake", flavor="gfortran")
#         == "hipsparse-gfortran-targets.cmake"
#     )
#     assert dh.clone_and_rename("eccodes.pc", flavor="gfortran") == "eccodes-gfortran.pc"
#     assert (
#         dh.clone_and_rename("/usr/lib/eccodes.pc", flavor="gfortran")
#         == "/usr/lib/eccodes-gfortran.pc"
#     )
#     assert (
#         dh.clone_and_rename("/usr/lib/eccodes.foo.pc", flavor="gfortran")
#         == "/usr/lib/eccodes.foo-gfortran.pc"
#     )


if __name__ == "__main__":
    import pytest

    pytest.main()



#!/usr/bin/python3
"""
Library  handling for Fortran  dh_fortran_lib

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

import os
import re
import click
from subprocess import check_output
from os.path import exists, basename, islink
import dhfortran.compilers as cmplrs
import dhfortran.debhelper as dh
import dhfortran.cli as cli
import magic
import fnmatch
from pathlib import Path

class Shlib:
    """
    Shared libs have a complex number of cases, so make a shlib object to
    # track all the info on a shlib
    """
    objs = {} # All known shlibs, key is orig 'libfoo'

    TYPE_UNKNOWN=0
    TYPE_NORMAL=1     # libfoo.so.2.1 etc
    TYPE_POSTFIX1=2   # libfoo-2.0.0.so
    TYPE_POSTFIX2=3   # libzzipmmapped-0.so.13.0.78
    TYPE_NOVER=4      # libfoo.so no soversion

    def __new__(cls , helper, orig_libfile: str, target_pkg, target_destdir):
        """ Creates or finds shlib object.
        Sets obj.name, other elements are set blank and filled by __init__"""
        # Insert abi into name
        flavor = helper.options.flavor
        abi = cmplrs.get_abi_vendor(flavor)
        
        p = Path(orig_libfile).resolve()
        name = p.name[:p.name.find('.so')]
        # reference. we may have multiple libraries with the same name, different
        # abis, and in different paths
        pn = f"{abi}:{target_destdir}:{name}"

        if pn in Shlib.objs:
            return Shlib.objs[pn]
        else:
            obj = object.__new__(cls)
            obj.flavor = flavor
            obj.abi = abi
            obj.name = name
            obj.orig_libfile = None
            obj.orig_libname = None
            obj.symlinks = []
            obj.libs_pkg = None
            obj.libdevel_pkg = None
            Shlib.objs[pn] = obj
            return obj

        
    def __init__(self, libfilehelper, orig_libfile: str, target_pkg, target_destdir):
        """
        """
        self.get_pkgs(libfilehelper, target_pkg)

        if self.libdevel_pkg == target_pkg:
            self.symlinks.append((orig_libfile,target_pkg,target_destdir))
            
        if islink(orig_libfile):
            self.symlinks.append((orig_libfile,target_pkg,target_destdir))
        else:
            self.orig_libfile = Path(orig_libfile).resolve()
            self.orig_libname = self.orig_libfile.name
            self.target_pkg = target_pkg
            self.target_destdir = target_destdir
            # Set kind,major,orig_soname,ext
            self.parse_shlib(orig_libfile)
        

    def get_pkgs(self, libfilehelper, target_pkg):
        """
        Each shlib belongs in 1 libs pkg, symlinks in 1 libdevel pkg
        """
        if len(libfilehelper.libs_pkgs) == 1:
            self.libs_pkg = libfilehelper.libs_pkgs[0]
        else:
            if target_pkg in libfilehelper.libs_pkgs:
                self.libs_pkg = target_pkg
        if len(libfilehelper.libdevel_pkgs) == 1:
            self.libdevel_pkg = libfilehelper.libdevel_pkgs[0]
        else:
            if target_pkg in libfilehelper.libdevel_pkgs:
                self.libdevel_pkg = target_pkg
        
    def parse_shlib(self, libfile: str):
        """ Helper. get soname from library. Passed real shlib
        set kind,major,orig_soname,ext
        """
            
        self.kind = Shlib.TYPE_UNKNOWN
        res = check_output(["objdump", "-p", libfile]).decode("utf-8")
        for line in res.split("\n"):
            lk = line.split()
            if len(lk) == 2 and lk[0] == "SONAME":
                soname=lk[1]
                self.orig_soname = soname
                if m:= re.search(r'-(\d+).so.(\d+)',soname):
                    # matches libfoo-0.so.13.1.2
                    self.kind=Shlib.TYPE_POSTFIX2
                    self.major = m.group(2)
                    self.ext = libfile[libfile.rfind(".so.")+3:]
                    return
                if m:= re.search(r'.so.(\d+(?:\.\d+)?)',soname):
                    # libpoints.so.1.2.3
                    self.kind = Shlib.TYPE_NORMAL
                    m2 = re.search(r'(\d+)', m.group(1))
                    self.major = m2.group(1)
                    self.ext = libfile[libfile.rfind(".so.")+3:]
                    return
                if m:= re.search(r'-(\d+\.\d+(?:\.\d+)?)',soname):
                    # libgctp-2.0.0.so
                    self.kind = Shlib.TYPE_POSTFIX1
                    sover = m.group(1)
                    self.major = sover[:sover.find('.')]
                    self.ext = ''
                    return
        # case libfoo.so, use this as soname.
        self.kind = Shlib.TYPE_NOVER
        self.major = None
        self.orig_soname = self.orig_libname
        self.ext = ''

        
    def set_new_soname_and_libname(self):
        """ Set the new soname for a file. In the case of no orig SO version,
        create a new soversion (self.major)
        Ditto for libname, libso
        """
        match self.kind:
            case Shlib.TYPE_NOVER: 
                # Ensure major is set
                self.major = '0d' # Default to a Debian 0d soversion
                if self.libs_pkg:
                    # is there a version postfix on the lib pkg name?
                    if m:=re.search(r'\d+$', self.libs_pkg):
                        self.major = m.group(0)
                self.ext = f".{self.major}"
                self.new_soname = f"{self.name}-{self.abi}.so.{self.major}"
                self.new_libname = f"{self.name}-{self.abi}.so{self.ext}"
                self.new_libso = f"{self.name}-{self.abi}.so"
            case Shlib.TYPE_POSTFIX1:
                # libgctp-2.0.0.so should become ligctp-gnu-2.0.0.so
                m= re.search(r'(.*)-(\d+(?:\.\d+)*)',self.orig_soname)
                self.new_soname = f"{m.group(1)}-{self.abi}-{m.group(2)}.so"
                self.new_libname = f"{m.group(1)}-{self.abi}-{m.group(2)}.so{self.ext}"
                self.new_libso = self.new_soname
            case Shlib.TYPE_POSTFIX2:
                # soname of the form libfoo-0.so.13  for libfoo-0.so.13.0.78
                m = re.search(r'(.*)-(\d+(?:\.\d+)*)',self.orig_soname)
                self.new_soname = f"{m.group(1)}-{self.abi}-{m.group(2)}.so.{self.major}"
                self.new_libname = f"{m.group(1)}-{self.abi}-{m.group(2)}.so{self.ext}"
                self.new_libso = f"{m.group(1)}-{self.abi}-{m.group(2)}.so"
            case _:
                assert self.major is not None
                prefix = self.orig_soname[: self.orig_soname.rfind(".so")]
                self.new_soname = f"{prefix}-{self.abi}.so.{self.major}"
                self.new_libname = f"{prefix}-{self.abi}.so{self.ext}"
                self.new_libso = f"{prefix}-{self.abi}.so"

    
    def process_shared_lib(self,helper, orig_libfile: str, target_pkg, target_destdir ):
        """ Called from process_file() multiple times for each lib or
            symlink we've been given
        """

        flibdir = f"/usr/lib/{cmplrs.multiarch}/"
        full_lib_path = f"debian/{self.libs_pkg}/{flibdir}"
        full_libdevel_path=f"debian/{self.libdevel_pkg}/{flibdir}"

        if not islink(orig_libfile):
            self.set_new_soname_and_libname()
            dh.install_dir(full_lib_path)
            
            helper.doit(["patchelf","--set-soname",self.new_soname,
                         "--output",f"{full_lib_path}/{self.new_libname}",
                         orig_libfile])
            if helper.needs_orig_libs():
                helper.doit(['cp',orig_libfile,f"{full_lib_path}"])
            dh.make_symlink(f"{full_lib_path}/{self.new_soname}",
                            f"{full_lib_path}/{self.new_libname}",                            
                            target_destdir)
                
        # If we know the new libname (processed lib already),
        # we can handle any symlinks we've been passed - add to dev pkg

        if 'new_libname' in self.__dict__:
            for (orig_libfile, link_target_pkg, link_target_dest) in self.symlinks:
                dh.install_dir(f"{full_libdevel_path}/fortran/{self.abi}")
                dh.make_symlink(f"{full_libdevel_path}/{self.new_libso}",
                                f"{full_libdevel_path}/{self.new_libname}",
                                target_destdir)
                dh.make_symlink(f"{full_libdevel_path}/fortran/{self.abi}/{self.name}.so",
                                f"{full_libdevel_path}/{self.new_libname}",
                                target_destdir)
                if helper.needs_orig_libs():
                    dh.make_symlink(f"{full_libdevel_path}/{self.name}.so",                                
                                    f"{full_libdevel_path}/{self.orig_libname}",
                                    target_destdir)
            self.symlinks = []

        

##
## Library-file -specific interface for debhelper
##
class LibFileHelper(dh.DhFortranHelper):

    def __init__(self, options):
        super().__init__(options, "fortran-lib", "dh_fortran_lib")

        self.remapped_libs = {}

    def needs_orig_libs(self) -> bool:
        """Do we need to ship the original libs, ie are any executables/libs linked
        against them ?"""
        if "DH_NO_ORIG_LIBS" in os.environ:
            return False
        return self.options.include_orig_libs

    
    def autogenerate_pkg_filelist(
        self, searchdirs: list[str], dest=None
    ) -> list[tuple[str, str, str]]:
        """For automatic mode, find  lib files to install"""

        worklist = []
        libs_pkg, libdevel_pkg = self.libs_pkgs[0], self.libdevel_pkgs[0]
        for d in searchdirs:
            if d is None:
                continue
            for root, dir, files in os.walk(d):
                if root.startswith("./debian"):
                    # Don't traverse down ./debian ( note './' to allow
                    # builddir=debian/tmp etc)
                    continue
                for item in fnmatch.filter(files, "*.a"):
                    f = f"{root}/{item}"
                    if "ar archive" in magic.from_file(f):
                        worklist.append((libdevel_pkg, f, dest))
                for item in fnmatch.filter(files, "*.so*"):
                    f = f"{root}/{item}"
                    if "ar archive" in magic.from_file(f):
                        print(f"DEBUG: adding ar archive {f} to {libs_pkg}")
                        worklist.append((libs_pkg, f, dest))
                        continue
                    if "shared object" in magic.from_file(f):
                        worklist.append((libs_pkg, f, dest))
                        continue
                    if item.endswith(".so"):
                        m=magic.from_file(f)
                        print(f"DEBUG {f=} {m=}")
                        if ("symbolic link" in m) or ("ar archive" in m):
                            print(f"DEBUG: adding {f}")
                            worklist.append((libdevel_pkg, f, dest))
        return worklist

    
    def can_work_automatically(self, *cmdline_files: list[str]) -> bool:
        """Should we automatically automagically detect lib files, or work on
        specified packages?
        Called by Debhelper.process_and_move_files()

        Run if we haven't run already, haven't been passed cmdline files,
        """
        if self.options.buildsystem != "fortran":
            cli.debug_print(
                "dh_fortran_lib : buildsystem not fortran; not running automatically"
            )
            return False
        if len(cmdline_files) > 0:
            cli.debug_print("dh_fortran_lib not automatic: files passed on cmdlline")
            return False
        for p in self.packages:
            if self.has_acted_on_package(p):
                cli.debug_print(
                    "dh_fortran_lib not automatic: has acted on pkgs already"
                )
                return False
        if len(self.config_file_pkgs) != 0:
            cli.verbose_print(
                "dh_fortran won't run automatically: user has given fortran-lib files"
            )
            return False
        if len(self.libdevel_pkgs) != 1 and len(self.libs_pkgs) != 1:
            cli.debug_print(
                "dh_fortran_lib: Need 1 lib, 1 libdevel pkg to run automatically"
            )
            return False
        cli.verbose_print(
            "No fortran-lib files found, 1 libdevel package, 1 libs pkg: "
            "may install libs automatically"
        )
        return True

    def compute_dest(self, libfile, target_pkg, target_dest=None) -> str:
        """What directory should a library be filed ?
        Should be called by base DebHelper() to move files
        _dest : directory (str)
        """
        abi = cmplrs.get_abi_vendor(self.options.flavor)
        flibdir = f"/usr/lib/{cmplrs.multiarch}/fortran/{abi}"
        if target_dest is None:
            return flibdir
        if target_dest.startswith("/"):
            return target_dest
        else:
            return f"{flibdir}/{target_dest}"

    def process_static_lib(self, orig_libfile, target_pkg, target_dest):
        libname = basename(orig_libfile)
        destfile = f"debian/{target_pkg}/{target_dest}/{libname}"
        if exists(destfile):
            if self.options.flavor == self.options.preferred:
                cli.verbose_print(f"{destfile} exists but will be overwritten"
                                  "(preferred)")
            else:
                cli.verbose_print(f"{destfile} exists and will not be overwritten")
                return

        dh.install_dir(f"debian/{target_pkg}/{target_dest}")
        self.doit(["cp", "--reflink=auto", "-a", orig_libfile, destfile])
        if (
            self.needs_orig_libs()
            and self.options.flavor == cmplrs.get_fc_default()
        ):
            destdir = f"debian/{target_pkg}/usr/lib/{cmplrs.multiarch}"
            dh.install_dir(destdir)
            self.doit(["cp", "--reflink=auto", "-a", orig_libfile, destdir])


    def process_file(
        self,
        pkg: str,
        libfile: str,
        target_pkg: str,
        target_destdir: str | None = None,
    ):
        """Called by Debhelper.process_and_move_files()"""
        cli.debug_print(
            f"process_file: name {libfile=}  {target_pkg=} {target_destdir=}"
        )
        destdir = self.compute_dest(libfile, target_pkg,  target_destdir)

        if libfile.endswith(".a"):
            self.process_static_lib(libfile, target_pkg, destdir)
            return
        if libfile.find(".so") == -1:
            cli.warning(f"file {libfile} is not an ELF file; ignoring")
            return
        # Handle shared lib
        s = Shlib(self, libfile, target_pkg, target_destdir)
        s.process_shared_lib(self, libfile, target_pkg, target_destdir)


@click.command(
    context_settings=dict(
        ignore_unknown_options=False,
    )
)
@click.option("--fc", help="Fortran compiler flavor (eg gfortran-14, flang-21)")
@click.option("--flavor", help="Fortran compiler flavor. DEPRECATED, use --fc instead")
@click.option(
    "--preferred", help="Preferred compiler flavor when installing, eg gfortran-15"
)
@click.option(
    "--include-orig-libs",
    help="Include original libs as well as moved versions",
    is_flag=True,
    default=True,
)
@click.argument("files", nargs=-1, type=click.UNPROCESSED)
@cli.debhelper_common_args
def dh_fortran_lib(files, *args, **kwargs):
    """
    *dh_fortran_lib* is a debhelper program that enables multiple compiler flavours
    of a Fortran library to be installed in parallel by mangling the library filename
    and SONAME.

    Fortran libraries compiled by different compilers are not expected to be
    ABI-compatible, and hence for multiple compilers to be supported simultaneously
    the libraries must be named differently,
    and shared libraries need to include the  compiler flavor in the SONAME.

    *dh_fortran_lib* makes this possible without changes being necessary to the upstream
    library code.

    It does this by renaming a library, for example:

       $(LIBDIR)/libfiat.so.1.2 => $(LIBDIR)/libfiat-flang.so.1.2
    =back

    Symlinks also get renamed:

       $(LIBDIR)/libfiat.so.1 => $(LIBDIR)/libfiat-flang.so.1

    A  compilation link is added per vendor /ABI :
    
       $(LIBDIR)/fortran/gfortran/libfiat.so -> $(LIBDIR)/libfiat-flang.so.1.2

    and the SONAME in the ELF file is changed:

       $ readelf -a $(LIBDIR)/libfiat.so.1.2 | grep SONAME
           0x000000000000000e (SONAME)             Library soname: [libfiat.so.1]
       $ readelf -a $(LIBDIR)/libfiat-flang.so.1.2 | grep SONAMEa
           0x000000000000000e (SONAME)             Library soname: [libfiat-flang.so.1]

    Note this is defined per ABI-vendor: if fc=gfortran-14, the link is named for 'gnu'.
    If the links are already defined, then the preferred flavor is examined. For each
    vendor, dh_fortran_lib has a preferred flavor: eg gfortran-14 or gfortran-15.
    If the flavor of the library provided is the preferred flavor, the library is
    replaced if it exists. The option --preferred can be used to override the default
    choice.

    For static libraries, we place them in a ABI-specific directory:

            $(LIBDIR)/fortran/gnu/libfiat.a

    The consequence of this is that any library that builds against I<libfiat> with
    appropriate search paths set will use *libfiat-gnu* instead. This enables parallel
    builds with multiple compiler flavors to be installed simultaneously.

    == USAGE

    The expected usage is that this will be called in debian/rules as:

        dh_fortran_lib --fc=$(FLAVOR) [--preferred=$(PREFERRED)] \
              $(BUILDDIR)/XXX/libfiat.so.1

    The files are installed in the tmpdir (usually debian/tmp) by default.

    When installing multiple flavors with the same ABI (eg gfortran-14, gfortran-15),
    you can use the --preferred option to state which library to install; in this case,
    if FLAVOR==PREFERRED, then
    this library is installed even if an existing library with the same ABI is present.
    """

    cli.debug_print(f"dh_fortran_lib called with files {files} kwargs {kwargs}")
    cli.validate_flavor(kwargs["flavor"])
    cli.validate_compiler(kwargs["fc"])
    fc, flavor = kwargs["fc"], kwargs["flavor"]
    if fc is not None:
        flavor = fc
    # Get defaults if not defined.
    flavor, arch = cmplrs.get_fc_flavor_arch(flavor)

    kwargs.update(
        {
            "flavor": flavor,
            "preferred": cmplrs.get_preferred(kwargs["preferred"], fc),
        }
    )

    d = LibFileHelper(dh.build_options(**kwargs))

    searchdirs = [d.options.builddir, d.options.tmpdir]
    d.process_and_move_files(searchdirs, *files)
    dh.install_dir("debian/.debhelper")
    with open("debian/.debhelper/remappped_fortran_libs", "wt+") as f:
        print(d.remapped_libs, file=f)


if __name__ == "__main__":
    import pytest

    pytest.main(["tests/lib.py"])



#!/usr/bin/python3
"""
Helper routines for dhfortran including inheriting from debhelper

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 GNU Public License.
"""

import os
import dhfortran.cli as cli
from subprocess import check_output
import errno
import re
from glob import glob
from os import makedirs, environ, unlink
from os.path import basename, exists, join, dirname, isdir, islink
from sys import argv
from dhfortran import DEPENDS_SUBSTVARS, PKG_NAME_TPLS

###
### COPIED FROM dh-python
###


parse_dep = re.compile(
    r"""[,\s]*
    (?P<name>[^\s:]+)(?::any)?
    \s*
    \(?(?P<version>([>=<]{2,}|=)\s*[^\)]+)?\)?
    \s*
    (?:\[(?P<arch>[^\]]+)\])?
    """,
    re.VERBOSE,
).match


def install_dir(dname: str):
    """mkdir -p dir in appropriate place"""
    cli.verbose_print(f"install_dir ({dname})")
    os.makedirs(dname, exist_ok=True)

        
def relpath(target, link):
    """Return relative path.

    >>> relpath('/usr/share/python-foo/foo.py', '/usr/bin/foo', )
    '../share/python-foo/foo.py'
    """
    t = target.split("/")
    lk = link.split("/")
    # remove any dup '//'
    while "" in t:
        t.remove("")
    while "" in lk:
        lk.remove("")
    while lk and lk[0] == t[0]:
        del lk[0], t[0]
    return "/".join([".."] * (len(lk) - 1) + t)

def make_symlink(src, dest, pkg=None):
        """make_symlink($dest, $src[, $pkg]) creates a symlink from  $dest -> $src.
        if $pkg is given, $dest will be created within it."""
        s = f"{pkg}/{src}" if pkg else src
        p = relpath(dest, src)
        if p == "":
            # src and dest are the same
            return
        cli.verbose_print(f"ln -s {s} {p}")
        if islink(s):  # So symlink doesn't fail on idempotent
            unlink(s)
        os.symlink(p, s)


# def move_file(fpath, dstdir):
#     """Move file to dstdir. Works with symlinks (including relative ones).
#     TODO: relative_symlink undefined; fn unused, delete? """
#     if isdir(fpath):
#         dname = split(fpath)[-1]
#         for fn in os.listdir(fpath):
#             move_file(join(fpath, fn), join(dstdir, dname))

#     if islink(fpath):
#         dstpath = join(dstdir, split(fpath)[-1])
#         relative_symlink(os.readlink(fpath), dstpath)
#         os.remove(fpath)
#     else:
#         os.rename(fpath, dstdir)


def build_options(**options):
    """Build an Options object from kw options"""
    default_options = {
        "arch": None,
        "package": [],
        "no_package": [],
        "match_packages": False,  # TODO Revist this decision
        "write_log": True,
        "remaining_packages": False,
        "sourcedir": None,
        "tmpdir": None,
        "DOPACKAGES": [],
    }
    if "arch" in options and options["arch"] is False:
        options["arch"] = None
    if "package" in options and isinstance(options["package"], str):
        options["package"] = [options["package"]]
    built_options = default_options
    built_options.update(options)
    return type("Options", (object,), built_options)


class DebHelper:
    """Reinvents the wheel / some dh functionality (Perl is ugly ;-P)"""

    def __init__(self, options, impl):
        self.options = options
        self.packages = {}
        self.build_depends = {}
        self.impl = impl
        skip_tpl = set()
        for name, tpls in PKG_NAME_TPLS.items():
            if name != impl:
                skip_tpl.update(tpls)
        skip_tpl = tuple(skip_tpl)
        substvar = DEPENDS_SUBSTVARS[impl]

        pkgs = options.package
        skip_pkgs = options.no_package

        try:
            with open("debian/control", "r", encoding="utf-8") as fp:
                paragraphs = [{}]
                field = None
                for lineno, line in enumerate(fp, 1):
                    if line.startswith("#"):
                        continue
                    if not line.strip():
                        if paragraphs[-1]:
                            paragraphs.append({})
                            field = None
                        continue
                    if line[0].isspace():  # Continuation
                        paragraphs[-1][field] += line.rstrip()
                        continue
                    if ":" not in line:
                        raise Exception(
                            "Unable to parse line %i in debian/control: %s"
                            % (lineno, line)
                        )
                    field, value = line.split(":", 1)
                    field = field.lower()
                    paragraphs[-1][field] = value.strip()
        except IOError:
            raise Exception("cannot find debian/control file")

        # Trailing new lines?
        if not paragraphs[-1]:
            paragraphs.pop()

        if len(paragraphs) < 2:
            raise Exception(
                "Unable to parse debian/control, found less than " "2 paragraphs"
            )

        self.source_name = paragraphs[0]["source"]

        build_depends = []
        for field in ("build-depends", "build-depends-indep", "build-depends-arch"):
            if field in paragraphs[0]:
                build_depends.append(paragraphs[0][field])
        build_depends = ", ".join(build_depends)
        for dep1 in build_depends.split(","):
            for dep2 in dep1.split("|"):
                details = parse_dep(dep2)
                if details:
                    details = details.groupdict()
                    if details["arch"]:
                        architectures = details["arch"].split()
                    else:
                        architectures = [None]
                    for arch in architectures:
                        self.build_depends.setdefault(details["name"], {})[arch] = (
                            details["version"]
                        )

        for paragraph_no, paragraph in enumerate(paragraphs[1:], 2):
            if "package" not in paragraph:
                raise Exception(
                    "Unable to parse debian/control, paragraph %i "
                    "missing Package field" % paragraph_no
                )
            binary_package = paragraph["package"]
            if skip_tpl and binary_package.startswith(skip_tpl):
                cli.verbose_print(f"skipping package: {binary_package}")
                continue
            # TODO. Expand this bit
            self.options.DOPACKAGES.append(binary_package)
            if pkgs and binary_package not in pkgs:
                continue
            if skip_pkgs and binary_package in skip_pkgs:
                continue
            if options.remaining_packages and self.has_acted_on_package(binary_package):
                continue
            if "package-type" in paragraph:
                typ = paragraph["package-type"]
            else:
                typ = "deb"
            if "section" in paragraph:
                section = paragraph["section"]
            else:
                section = None
            pkg = {
                "substvars": {},
                "autoscripts": {},
                "arch": paragraph["architecture"],
                "type": typ,
                "section": section,
            }
            if (
                options.arch is False
                and pkg["arch"] != "all"
                or options.arch is True
                and pkg["arch"] == "all"
            ):
                # TODO: check also if arch matches current architecture:
                continue

            if options.match_packages:
                if not binary_package.startswith(PKG_NAME_TPLS[impl]):
                    # package doesn't have common prefix (libfortran-)
                    # so lets check if Depends/Recommends contains the
                    # appropriate substvar
                    if substvar not in paragraph.get(
                        "depends", ""
                    ) and substvar not in paragraph.get("recommends", ""):
                        cli.verbose_print(
                            f"skipping {binary_package}: (missing {substvar})"
                        )
                        continue

            # Operate on binary_package
            self.packages[binary_package] = pkg

        fp.close()
        cli.verbose_print(
            f"source={self.source_name}, binary packages={list(self.packages.keys())}"
        )

    def has_acted_on_package(self, package):
        try:
            with open(f"debian/{package}.debhelper.log", encoding="utf-8") as f:
                for line in f:
                    if line.strip() == self.tool_name:
                        return True
        except IOError as e:
            if e.errno != errno.ENOENT:
                raise
        return False

    # Policy says that if the link is all within one toplevel
    # directory, it should be relative. If it's between
    # top level directories, leave it absolute.
    # if not self.options.no_act:
    #    relative_symlink(dest, src)

    def find_files(self, searchdirs, pkg, dest, *files):
        """Search for each file in searchdirs, return list of tuples (pkg,file, dest)
        Recurse into directories"""
        result = []
        for f in files:
            if not f:
                continue
            matches = []
            for s in searchdirs:
                if s is None:
                    continue
                nm = f[1:] if f.startswith("/") else f
                for p in glob(nm, root_dir=s):
                    if isdir(f"{s}/{p}"):
                        subdest = basename(p) if dest == "" else f"{dest}/{basename(p)}"
                        x = self.find_files(pkg, subdest, [f"{s}/{p}"], "*")
                        matches += x
                    else:
                        matches.append((pkg, f"{s}/{p}", dest))
            if matches:
                result += matches
            else:
                cli.warning(f"{self.tool_name}: missing file(s) {f}")
        return result

    def addsubstvar(self, package, name, value):
        """debhelper's addsubstvar"""
        self.packages[package]["substvars"].setdefault(name, []).append(value)

    def autoscript(self, package, when, template, args):
        """debhelper's autoscript"""
        self.packages[package]["autoscripts"].setdefault(when, {}).setdefault(
            template, []
        ).append(args)

    def complex_doit(self, *args, **kwargs):
        """Same as complex_doit() from Debian::Debhelper::Dh_Lib;"""
        cli.verbose_print(f"complex_doit({args},{kwargs})")
        if self.options.no_act:
            return
        # TODO

    def doit(self, *args, **kwargs):
        """Execute a command given as a list"""
        cli.verbose_print(f"doit({args},{kwargs})")
        if self.options.no_act:
            return
        check_output(*args)
        # TODO
        
    def is_udeb(self, package):
        return self.packages[package]["type"] == "udeb"


    def save_autoscripts(self):
        for package, settings in self.packages.items():
            autoscripts = settings.get("autoscripts")
            if not autoscripts:
                continue

            for when, templates in autoscripts.items():
                fn = "debian/%s.%s.debhelper" % (package, when)
                if exists(fn):
                    with open(fn, "r", encoding="utf-8") as datafile:
                        data = datafile.read()
                else:
                    data = ""

                new_data = ""
                for tpl_name, args in templates.items():
                    for i in args:
                        # try local one first (useful while testing dh_python3)
                        fpath = join(
                            dirname(__file__), "..", "autoscripts/%s" % tpl_name
                        )
                        if not exists(fpath):
                            fpath = "/usr/share/debhelper/autoscripts/%s" % tpl_name
                        with open(fpath, "r", encoding="utf-8") as tplfile:
                            tpl = tplfile.read()
                        if self.options.compile_all and args:
                            # TODO: should args be checked to contain dir name?
                            tpl = tpl.replace("-p #PACKAGE#", "")
                        elif settings["arch"] == "all":
                            tpl = tpl.replace("#PACKAGE#", package)
                        else:
                            arch = environ["DEB_HOST_ARCH"]
                            tpl = tpl.replace("#PACKAGE#", "%s:%s" % (package, arch))
                        tpl = tpl.replace("#ARGS#", i)
                        if tpl not in data and tpl not in new_data:
                            new_data += "\n%s" % tpl
                if new_data:
                    data += "\n# Automatically added by {}".format(
                        basename(argv[0])
                    ) + "{}\n# End automatically added section\n".format(new_data)
                    with open(fn, "w", encoding="utf-8") as fp:
                        fp.write(data)

    def save_substvars(self):
        for package, settings in self.packages.items():
            substvars = settings.get("substvars")
            if not substvars:
                continue
            fn = "debian/%s.substvars" % package
            if exists(fn):
                with open(fn, "r", encoding="utf-8") as datafile:
                    data = datafile.read()
            else:
                data = ""
            for name, values in substvars.items():
                p = data.find("%s=" % name)
                if p > -1:  # parse the line and remove it from data
                    e = data[p:].find("\n")
                    line = data[p + len("%s=" % name) : p + e if e > -1 else None]
                    items = [i.strip() for i in line.split(",") if i]
                    if e > -1 and data[p + e :].strip():
                        data = "%s\n%s" % (data[:p], data[p + e :])
                    else:
                        data = data[:p]
                else:
                    items = []
                for j in values:
                    if j not in items:
                        items.append(j)
                if items:
                    if data:
                        data += "\n"
                    data += "%s=%s\n" % (name, ", ".join(items))
            data = data.replace("\n\n", "\n")
            if data:
                with open(fn, "w", encoding="utf-8") as fp:
                    fp.write(data)

    def pkgfile(self, package: str, suffix: str) -> str | None:
        """Incomplete clone of /usr/share/perl5/Debian/Debhelper/Dh_Lib.pm version"""
        hostarch, hostos = "FIXME", "FIXME"
        searchpath = [
            f"debian/{package}.{suffix}.{hostarch}",
            f"debian/{package}.{suffix}.{hostos}",
            f"debian/{package}.{suffix}",
            f"debian/{suffix}",
        ]
        for s in searchpath:
            if exists(s):
                return s
        return None

    def generated_file(self, package: str, filename: str):
        # Generated files are cleaned by dh_clean AND dh_prep
        # - Package can be set to "_source" to generate a file relevant
        #   for the source package (the meson build does this atm.).
        #   Files for "_source" are only cleaned by dh_clean.
        dirname = f"debian/.debhelper/generated/{package}"
        makedirs(dirname, exist_ok=True)
        pth = f"{dirname}/{filename}"
        return pth

    def log_installed_files(self, package, installed, exclude_patterns=[]):
        if self.options.no_act:
            return
        if installed == []:
            return
        log = self.generated_file(package, f"installed-by-{self.tool_name}")
        with open(log, "a") as f:
            for src in installed:
                print(src, file=f)

    def skip_package(self, pkg) -> bool:
        """Do we skip this package or process it ?"""
        if self.options.package is not None and pkg not in self.options.package:
            return True
        if self.is_udeb(pkg):
            return True
        if pkg not in self.options.DOPACKAGES:
            return True
        if self.options.firstpackage and pkg != self.options.firstpackage:
            cli.debug_print(
                f"checking package FIRSTPACKAGE {self.options.firstpackage} only "
            )
        return False

    def generate_pkg_filelist_from_cfgfile(
        self, searchdirs: list[str], pkg: str
    ) -> list[tuple[str, str, str]]:
        """Generate a list of files to work on
        @return list of tuples (package, filename, dest_dir)
        """

        def expand_variables(bits: list[str]) -> list[str]:
            """Helper fn. Do variable expansion on filepaths"""
            new_bits = []
            for fpath in bits:
                new_fpath = []
                for elm in fpath.split("/"):
                    if elm.startswith("$"):
                        # TODO
                        pass
                    new_fpath.append(elm)
                new_bits.append("/".join(new_fpath))
            return new_bits

        # Reads in the specified file, one line at a time. splits on words
        # If a value is passed in as the second parameter, then glob
        # expansion is done in the directory specified by the parameter ("." is
        # frequently a good choice).
        # TODO: In compat  13+, it will do variable expansion (after splitting the lines
        # into words)
        worklist = []
        cfgfile = self.packages[pkg]["config_file"]
        with open(cfgfile, "r") as f:
            lines = f.readlines()
        if lines[0].startswith("#!"):
            # Some config files are shell scripts that generate output, so ...
            lines = check_output(["./" + cfgfile]).decode("utf-8").split("\n")
        for lineno, line in enumerate(lines, 1):
            if line == "" or line.startswith("#"):
                continue  # fragile
            bits = line.strip().split()
            bits = expand_variables(bits)
            dest = bits[1] if len(bits) > 1 else ""
            try:
                worklist += self.find_files(searchdirs, pkg, dest, bits[0])
            except Exception:
                badline = f"Bad line {lineno} in {cfgfile}"
                cli.warning(badline)
        # TODO finish error parsing
        return worklist

    def process_and_move_files(self, search_dirs, *cmdline_files):
        """The bulk of the work.
        subclass provides methods can_work_automatically(),
           autogenerate_pkg_filelist(), process_file()
        """

        # Worklist: list of (pkg, file, dest) tuples
        worklist = []
        if self.can_work_automatically(*cmdline_files):
            worklist += self.autogenerate_pkg_filelist(search_dirs)
        else:
            for pkg in self.config_file_pkgs:
                if self.skip_package(pkg):
                    continue
                worklist += self.generate_pkg_filelist_from_cfgfile(search_dirs, pkg)
            if len(cmdline_files) > 0:
                # TODO . glob breakdown
                pkg = self.options.package
                if pkg is not None and not self.skip_package(pkg):
                    for f in cmdline_files:
                        worklist += self.find_files(search_dirs, pkg, "", f)

        # Look at the install files for all packages to handle
        # list-missing/fail-missing, but skip really installing for
        # packages that are not being acted on.

        # TODO Skip excluded patterns in install files

        for pkg, fil, dest in worklist:
            self.log_installed_files(pkg, [fil])
            self.process_file(pkg, fil, pkg, dest)

        if not self.options.write_log:
            return
        for package, _ in self.packages.items():
            with open(f"debian/{package}.debhelper.log", "a", encoding="utf-8") as f:
                f.write(self.tool_name + "\n")

    def save(self):
        self.save_substvars()
        self.save_autoscripts()
        self.save_log()


class DhFortranHelper(DebHelper):
    """dhfortran-specific stuff"""

    def __init__(self, options, suffix: str, tool_name: str):
        # TODO rework these
        self.command = {"fortran": "dh-fortran"}
        self.tool_name = tool_name
        self.suffix = suffix
        super().__init__(options, "fortran")
        for pkg in self.packages:
            config_file = self.pkgfile(pkg, self.suffix)
            self.packages[pkg]["config_file"] = config_file
        self.libs_pkgs = [
            p for p in self.packages if self.packages[p]["section"] == "libs"
        ]
        self.libdevel_pkgs = [
            p for p in self.packages if self.packages[p]["section"] == "libdevel"
        ]
        self.config_file_pkgs = [
            p for p in self.packages if self.packages[p]["config_file"]
        ]


if __name__ == "__main__":
    import pytest

    pytest.main(["tests/debhelper.py"])



#!/usr/bin/python3
"""
Test Driver for dh-fortran/main.py

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

from click.testing import CliRunner
import dhfortran.main as main
import dhfortran.debhelper as dh


def test_dh_fortran():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, [])
    assert result.exit_code == 0
    # assert result.output == 'Hello Peter!\n'


def test_dh_fortran_get_fc_default_simple():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fc_default"])
    assert result.exit_code == 0
    assert result.output == "gfortran-15\n"

    
def test_dh_fortran_get_fc_default_flang():
    runner = CliRunner()
    runner.env["FC"] = "flang-21"
    result = runner.invoke(main.dh_fortran, ["get_fc_default"])
    assert result.exit_code == 0
    assert result.output == "flang-21\n"

def test_dh_fortran_get_fc_default_flang_flags():
    runner = CliRunner()
    runner.env["FC"] = "flang-new-21"
    result = runner.invoke(main.dh_fortran, ["get_fc_default", "--fc","gfortran-15"])
    assert result.exit_code == 0
    assert result.output == "gfortran-15\n"

def test_dh_fortran_get_fc_optional_env():
    """ get_fc_optional : env overrides autodetect """
    runner = CliRunner()
    runner.env["FC_OPTIONAL"]= "flang-21 gfortran-14"
    result = runner.invoke(main.dh_fortran, ["get_fc_optional"])
    assert result.exit_code == 0
    assert result.output=="flang-21 gfortran-14\n"

    
def test_dh_fortran_get_fc_optional_flags():
    """ get_fc_optional : remove FC passed by flag
    TODO: How to pass empty ? """
    runner = CliRunner()
    runner.env["FC_OPTIONAL"]= "flang-21 gfortran-14"
    result = runner.invoke(main.dh_fortran, ["get_fc_optional","--fc","flang-21"])
    assert result.exit_code == 0
    assert result.output=="gfortran-14\n"

def test_dh_fortran_get_fc_flags():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fc_flags"])
    assert result.exit_code == 0
    assert (
        result.output
        == "-g -O2 -ffile-prefix-map=/home/alastair/git/dh-fortran/dhfortran=. -fstack-protector-strong -fstack-clash-protection -fcf-protection\n"
    )

def test_dh_fortran_get_fc_flags_flang21():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fc_flags","--fc","flang-21"])
    assert result.exit_code == 0


def test_dh_fortran_get_bad_fc():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fc_flags","--fc","garbage"])
    assert result.exit_code == 2
    assert result.output == """Usage: dh-fortran get_fc_flags [OPTIONS]
Try 'dh-fortran get_fc_flags --help' for help.

Error: fc compiler garbage is broken; bad symlink?
"""

def test_dh_fortran_get_abi_vendor():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_abi_vendor"])
    assert result.exit_code == 0
    assert result.output == "gnu\n"
    
def test_dh_fortran_get_abi_vendor_fc_gfortran():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_abi_vendor", "--fc","gfortran"])
    assert result.exit_code == 0
    assert result.output == "gnu\n"
    
def test_dh_fortran_get_fc_exe():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fc_exe"])
    assert result.exit_code == 0
    # assert result.output.endswith("gfortran-15\n")


def test_dh_fortran_get_fc_exe_env():
    runner = CliRunner()
    runner.env["FC"] = "flang-21"
    result = runner.invoke(main.dh_fortran, ["get_fc_exe"])
    assert result.exit_code == 0
    # assert result.output == "flang-21\n"


def test_dh_fortran_get_fc_exe_param():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fc_exe", "--flavor", "flang21"])
    assert result.exit_code == 0
    # assert result.output == "flang-new-21\n"


def test_dh_fortran_get_fc_exe_param():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fc_exe", "--flavor", "flangX"])
    # assert result.exit_code == 2


def test_dh_fortran_get_fmoddir():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fmoddir"])
    assert result.exit_code == 0
    assert result.output.endswith("/gfortran-mod-16\n")

def test_dh_fortran_get_fmoddir_fc():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fmoddir", "--fc", "flang-21"])
    assert result.exit_code == 0
    assert result.output.endswith("/flang-mod-1\n")

def test_dh_fortran_get_fmoddir_fc_prepend():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fmoddir", "--fc", "flang-21", "-I"])
    assert result.exit_code == 0
    assert result.output.startswith("-I")

def test_dh_fortran_get_fmoddirs():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fmoddir"])
    assert result.exit_code == 0
    #assert result.output == "gfortran-mod-16\n"

def test_dh_fortran_get_fmoddirs_prepend():
    # dh_fortran get_fmoddirs --fc gfortran-7 --prepend
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["get_fmoddir", "--fc", "gfortran-15", "--prepend"])
    assert result.exit_code == 0
    #assert result.output == "gfortran-mod-16\n"

    
def test_dh_fortran_clean():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["clean"])
    assert result.exit_code == 0


def test_dh_fortran_configure():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["configure"])
    assert result.exit_code == 0


def test_dh_fortran_build():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["build"])
    assert result.exit_code == 0


def test_dh_fortran_install():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["install"])
    assert result.exit_code == 0


def test_dh_fortran_test():
    runner = CliRunner()
    result = runner.invoke(main.dh_fortran, ["test"])
    assert result.exit_code == 0


if __name__ == "__main__":
    import pytest

    pytest.main()



#!/usr/bin/python3
"""
Test Driver for Shlib parsing in lib.py

Copyright (C) 2026 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

from click.testing import CliRunner
import dhfortran.lib as lib
from dhfortran.compilers import multiarch
import dhfortran.debhelper as dh
import dhfortran.tests.helpers as helpers
import pytest
import os


@pytest.fixture()
def set_env():
    os.environ["NO_ACT"] = "1"


class DummyHelper1:
    libs_pkgs = [ "libfoo1"]
    libdevel_pkgs = [ "libfoo-dev" ]

    def __init__(self):
        self.options = dh.build_options()
        self.options.flavor = "gfortran"

class DummyHelper2:
    libs_pkgs = [ 'libfoo1', 'libbar1' ]
    libdevel_pkgs = [ 'libfoo-dev' ]

    def __init__(self):
        self.options = dh.build_options()
        self.options.flavor = "gfortran"


def test_parse_shlib_normal1():
    """ Test shlib initial object and parsing """

    # standard library type
    libdir= f"/usr/lib/{multiarch}"    
    s = lib.Shlib(DummyHelper1(), f"{libdir}/libm.so.6", "libdummy-dev", "")
    assert s.kind == lib.Shlib.TYPE_NORMAL
    assert s.major == '6'
    assert s.ext == ".6"
    assert s.orig_soname == 'libm.so.6'

def test_parse_shlib_type_normal():
    """ 'Normal' type, with libfoo.so.1.2 or libfoo.so.1 naming, SOVER defined (TYPE_NORMAL)
    """
    lib.Shlib.objs = {} # reset
    s = lib.Shlib(DummyHelper1(), "dhfortran/tests/test01/libpoints.so.1.2", "libdummy-dev", "")
    assert s.kind == lib.Shlib.TYPE_NORMAL
    assert s.major == '1'
    assert s.ext == '.1.2'
    assert s.orig_soname == 'libpoints.so.1'

def test_parse_shlib_nover():
    """ Can we correcltly parse an SO with no versioning? (TYPE_NOVER)
    """
    # Nover library
    lib.Shlib.objs = {} # reset
    s = lib.Shlib(DummyHelper1(), "dhfortran/tests/test03/libpoints.so", "libpoints-dev", "")
    assert s.kind == lib.Shlib.TYPE_NOVER
    assert s.major == None
    assert s.ext == ''
    assert s.orig_soname == 'libpoints.so'

def test_parse_shlib_postfix1():
    """ gctp-type: libgctp-2.0.0.so (TYPE_POSTFIX1)
    """
    lib.Shlib.objs = {} # reset
    s = lib.Shlib(DummyHelper1(), f"dhfortran/tests/test04/libpoints-1.2.3.so", "libgctp-dev", "")    
    assert s.kind == lib.Shlib.TYPE_POSTFIX1
    assert s.major == '1'
    assert s.ext == ''
    assert s.orig_soname == 'libpoints-1.2.3.so'
    
def test_parse_shlib_postfix2():
    """" libzzipmmapped-0.so.13.0.78 type (TYPE_POSTFIX2)
    """
    lib.Shlib.objs = {} # reset
    s = lib.Shlib(DummyHelper1(), f"dhfortran/tests/test05/libpoints-0.so.13.0.78", "libdummy-dev", "")
    assert s.kind == lib.Shlib.TYPE_POSTFIX2
    assert s.major == '13'
    assert s.ext == '.13.0.78'
    assert s.orig_soname == 'libpoints-0.so.13'
    
def test_new_soname_normal1():
    """ Test the setting of a new soname """
    libdir= f"/usr/lib/{multiarch}"    
    lib.Shlib.objs = {} # reset
    s = lib.Shlib(DummyHelper1(), f"{libdir}/libm.so.6", "libpoints-dev", "")
    s.set_new_soname_and_libname()
    assert s.new_soname == 'libm-gnu.so.6'

def test_new_soname_normal2():
    lib.Shlib.objs = {} # reset
    s = lib.Shlib(DummyHelper1(), "dhfortran/tests/test01/libpoints.so.1.2", "libdummy-dev", "")
    s.set_new_soname_and_libname()
    assert s.new_soname == 'libpoints-gnu.so.1'
        
def test_new_soname_postfix1():
    lib.Shlib.objs = {} # reset
    s = lib.Shlib(DummyHelper1(), f"dhfortran/tests/test04/libpoints-1.2.3.so", "libpoints-dev", "")
    s.set_new_soname_and_libname()
    assert s.new_soname == 'libpoints-gnu-1.2.3.so'

def test_new_soname_postfix2():
    # Postfix wierd case (2)
    lib.Shlib.objs = {} # reset
    s = lib.Shlib(DummyHelper1(), f"dhfortran/tests/test05/libpoints-0.so.13.0.78", "libpoints-dev", "")
    s.set_new_soname_and_libname()    
    assert s.new_soname == "libpoints-gnu-0.so.13"

def test_new_soname_nover1():
    # Non-versioned library
    lib.Shlib.objs = {} # reset
    s = lib.Shlib(DummyHelper1(), "dhfortran/tests/test03/libpoints.so", "libpoints-dev", "")
    s.set_new_soname_and_libname()
    assert s.new_soname == 'libpoints-gnu.so.1' # guessed from libfoo1

def test_new_soname_nover2():
    # non-versioned, no hint in libname
    d = DummyHelper1()
    d.libs_pkgs = []
    lib.Shlib.objs = {} # reset
    s = lib.Shlib(d, "dhfortran/tests/test03/libpoints.so", "libpoints-dev", "")
    s.set_new_soname_and_libname()
    assert s.new_soname == 'libpoints-gnu.so.0d' # guessed
    
    
if __name__ == "__main__":
    import pytest

    pytest.main()



#!/usr/bin/python3
"""
CMake fragment handling for Fortran : tests

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

from click.testing import CliRunner
from dhfortran.cmake import dh_fortran_cmake
import dhfortran.debhelper as dh
import dhfortran.tests.helpers as helpers


# def test_rename(set_env):
#     assert (
#         dh.clone_and_rename("hipsparse-targets.cmake", vendor="gnu")
#         == "hipsparse-targets-gnu.cmake"
#     )
#     assert (
#         dh.clone_and_rename("hipsparse-targets.cmake", flavor="gfortran")
#         == "hipsparse-targets-gfortran.cmake"
#     )


def test_dh_fortran_cmake_trivial():
    runner = CliRunner()
    result = runner.invoke(dh_fortran_cmake, [])
    assert result.exit_code == 0
    assert result.output == ""


def test_dh_fortran_cmake_with_fc():
    runner = CliRunner()
    result = runner.invoke(dh_fortran_cmake, ["--fc", "gfortran-15"])
    assert result.exit_code == 0
    # assert result.output == 'Hello Peter!\n'


def test_dh_fortran_cmake_trivial_verbose():
    runner = CliRunner()
    result = runner.invoke(dh_fortran_cmake, ["--verbose"])
    assert result.exit_code == 0
    # assert result.output.startswith("dh_fortran_cmake called with")


# TODO Need better error output
def test_dh_fortran_cmake_with_garbage_fc():
    runner = CliRunner()
    result = runner.invoke(dh_fortran_cmake, ["--fc", "garbage"])
    assert result.exit_code == 2


#    assert (
#        result.output
#        == "Usage: wrapper [OPTIONS]\nTry 'wrapper --help' for help.\n\nError: garbage is not a valid compiler flavor\n"
#    )


def test_dh_fortran_cmake_list_of_files():
    runner = CliRunner()
    result = runner.invoke(dh_fortran_cmake, ["--verbose", "filea", "fileb"])
    assert result.exit_code == 0


#    assert result.output.startswith("dh_fortran_cmake called with")


if __name__ == "__main__":
    import pytest

    pytest.main()
